/******************************************************************************
 * FLUID (Flash Loader Utility Independent of Device)
 *
 * (C) Delta Technologies 2001.
 * Cleanup, modifications and extensions by Mads Meisner-Jensen, mmj@ti.com.
 *
 * AMD and SST Flash Erase and Program Method
 *
 * $Id: amd.c 1.7.1.7 Mon, 28 Apr 2003 08:49:16 +0200 tsj $
 *
 ******************************************************************************/

#include "target.h"

//#define DEBUG_PROGRAM
//#define DEBUG_ERASE


static int amd_init(uint32 addr);
static int amd_erase(bss_t *bss, uint32 addr);
static int amd_program(bss_t *bss, uint16 *dst, uint16 *buf, int size);

const flash_t my_function_table = {
    amd_init,
    amd_erase,
    amd_program,
    0,
    0,
    0,
    0,
    0xAABBCCDD
};


/******************************************************************************
 * Main
 ******************************************************************************/

static int amd_init(uint32 addr)
{
    //my_addr = addr;

    return PROTO_READY;
}

// Status bits for AMD flash devices
#define AMD_DATA_POLLING          (0x80) /* DQ7 */
#define AMD_EXCEEDED_TIMING_LIMIT (0x20) /* DQ5 */


static int amd_erase(bss_t *bss, uint32 addr)
{
    uint16 status;
    int result;

#ifdef DEBUG_ERASE
    DEBUG_PUTSTR("E");
    DEBUG_PUTNUM((uint32) addr);
#endif

    FLASH_WRITE (bss->hexAAAA, 0xAA); // AMD Unlock cycle 1
    FLASH_WRITE (bss->hex5555, 0x55); // AMD Unlock cycle 2
    FLASH_WRITE (bss->hexAAAA, 0x80); // AMD Erase Unlock
    FLASH_WRITE (bss->hexAAAA, 0xAA); // AMD Unlock cycle 1
    FLASH_WRITE (bss->hex5555, 0x55); // AMD Unlock cycle 2
#if defined(SST)
    FLASH_WRITE (addr,   0x50); // SST Block Erase command
#else
    FLASH_WRITE (addr,   0x30); // AMD Sector Erase command
#endif

    // Erase is complete when the AMD_DATA_POLLING bit is set
    do {
        if ((result = bss->recv(bss)) != 0)
            return result;
        status = FLASH_READ(addr);
    }
    while (!(status & AMD_DATA_POLLING) &&
           !(status & AMD_EXCEEDED_TIMING_LIMIT));

    if (!(status & AMD_DATA_POLLING))
        result = PROTO_ERROR_FLASH_TIMEOUT;
    else
        result = 0;

    return result;
}

static int amd_program(bss_t *bss, uint16 *dst, uint16 *buf, int size)
{
    uint16 status;
    uint16 *dst_start = dst;
    int    result = 0;

#ifdef DEBUG_PROGRAM
    DEBUG_PUTSTR("P");
    DEBUG_PUTNUM((uint32) dst);
    DEBUG_PUTNUM((uint32) buf);
    DEBUG_PUTNUM((uint32) size);
#endif

#if !defined(SST)
    // SST flash devices do not support 'Unlock Bypassing'
    FLASH_WRITE(bss->hexAAAA, 0xAA); // AMD Unlock cycle 1
    FLASH_WRITE(bss->hex5555, 0x55); // AMD Unlock cycle 2
    FLASH_WRITE(bss->hexAAAA, 0x20); // AMD Unlock Bypass command
#endif

    while (result == 0 && size--)
    {
#if defined(SST)
        FLASH_WRITE(bss->hexAAAA, 0xAA); // AMD Unlock cycle 1
        FLASH_WRITE(bss->hex5555, 0x55); // AMD Unlock cycle 2
#endif

        FLASH_WRITE(bss->hexAAAA, 0xA0); // AMD program command (addr is dont-care)
        FLASH_WRITE(dst,   *buf);

        // Poll for completion, then check result.
        do {
            if ((result = bss->recv(bss)) != 0)
                return result;
            status = FLASH_READ(dst);
        } while (((status & 0x80) != (*buf & 0x80)) &&
                 !(status & AMD_EXCEEDED_TIMING_LIMIT));

        status = FLASH_READ(dst);
        if ((status & 0x80) != (*buf & 0x80))
            result = PROTO_ERROR_FLASH_TIMEOUT;

        dst++;
        buf++;
    }

    // Write the unlock bypass reset command to return to read array mode.
    FLASH_WRITE(dst_start, 0x90); // Unlock Bypass Reset
    FLASH_WRITE(dst_start, 0x00); // Unlock Bypass Reset (addr is dont-care)

    return result;
}
// Assembly code that satisfies compiler's need of an indirect call
asm("IND$CALL:");
asm("        bx      r4");
