/******************************************************************************
 * FLUID (Flash Loader Utility Independent of Device)
 *
 * (C) Delta Technologies 2001.
 * Cleanup, modifications and extensions by Mads Meisner-Jensen, mmj@ti.com.
 *
 * Intel Flash Erase and Program Method
 *
 * $Id: intel.c 1.11 Mon, 21 Oct 2002 18:39:13 +0200 mmj $
 *
 ******************************************************************************/

#include "target.h"

//#define DEBUG_PROGRAM
//#define DEBUG_ERASE


static int intel_bw_init(uint32 addr);
static int intel_bw_erase(struct bss_s *bss, uint32 addr);
static int intel_bw_program(struct bss_s *bss, uint16 *dst, uint16 *buf, int size);

const flash_t my_function_table = {
    intel_bw_init,
    intel_bw_erase,
    intel_bw_program,
    0,
    0,
    0,
    0,
    0xAABBCCDD
};


/******************************************************************************
 * Main
 ******************************************************************************/

static int intel_bw_init(uint32 addr)
{
    return PROTO_READY;
}


// Status bits for Intel flash memory devices
#define INTEL_STATE_MACHINE_DONE     (1<<7)
#define INTEL_BLOCK_ERASE_ERROR      (1<<5)
#define INTEL_CMD_SEQ_ERROR          ((1<<4)|(1<<5))
#define INTEL_PROGRAMMING_ERROR      (1<<4)
#define INTEL_VPP_RANGE_ERROR        (1<<3)
#define INTEL_BLOCK_LOCKED_ERROR     (1<<1)


static int intel_bw_erase(struct bss_s *bss, uint32 addr)
{
    int result;
    uint16 status;

#ifdef DEBUG_ERASE
    DEBUG_PUTSTR("E");
    DEBUG_PUTNUM((uint32) addr);
#endif

    FLASH_WRITE(addr, 0x50); // Intel Clear Status Register Command
    FLASH_WRITE(addr, 0xFF); // Intel Read Array Command

    do {
        FLASH_WRITE(addr, 0x60); // Intel Config Setup Command
        FLASH_WRITE(addr, 0xD0); // Intel Unlock Block Command
        FLASH_WRITE((addr + 0x04), 0x90); // Intel Read ID Plane Command
        status = FLASH_READ(addr + 0x04); // Locking changed?
    } while (status & 0x03);
    FLASH_WRITE(addr, 0xFF); // Intel Read Array Command

    FLASH_WRITE(addr, 0x50); // Intel Clear Status Register Command
    FLASH_WRITE(addr, 0x20); // Intel Erase Setup Command
    FLASH_WRITE(addr, 0xD0); // Intel Erase Confirm Command
    
    do {
        if ((result = bss->recv(bss)) != 0) {
            FLASH_WRITE(addr, 0xFF); // Intel Read Array Command
            return result;
        }
        status = FLASH_READ(addr);
    } while (!(status & INTEL_STATE_MACHINE_DONE));

    status = FLASH_READ(addr);

    if (status & INTEL_VPP_RANGE_ERROR)
        result = PROTO_ERROR_FLASH_VPP;
    else if ((status & INTEL_CMD_SEQ_ERROR) == INTEL_CMD_SEQ_ERROR)
        result = PROTO_ERROR_FLASH_COMMAND;
    else if (status & INTEL_BLOCK_ERASE_ERROR)
        result = PROTO_ERROR_FLASH_TIMEOUT;
    else if (status & INTEL_BLOCK_LOCKED_ERROR)
        result = PROTO_ERROR_FLASH_LOCKED;
    else 
        result = 0;

    FLASH_WRITE(addr, 0xFF); // Intel Read Array Command

#ifdef DEBUG_ERASE
    DEBUG_PUTNUM((uint32) status);
    DEBUG_PUTNUM((uint32) result);
#endif

    return result;
}

int status_check(uint16 *dst)
{
    int status, result;

        
    return result; 
}


#if 0
// Code to query 'buffer size' (from FFS), needs some porting before it will work with fluid. I expect this code to be put into intel_init()

// FIXME Also make it AMD compatible
void ffsdrv_query_cfi_data(uint16 flash_base_addr, uint16 addr, uint16* data)
{
    FLASH_WRITE_HALFWORD((0x55 | flash_base_addr), 0x98); // Set CFI query mode

    *data = FLASH_READ_HALFWORD(addr | flash_base_addr); // Query data

    FLASH_WRITE_HALFWORD (flash_base_addr, 0xFF);  // Intel read-array/reset command
}

// Funny looking code because it is copied and executed from RAM
myfp = (pf_t) (((int) &ffsdrv_query_cfi_data & 1) | (int) cfi_code);
(*myfp)(flash_base_addr, 0x2A<<1, &dev.write_buffersize);
#endif


// words
//#define FLASH_BUF_SIZE 32

// Intel Strata Buffer write function. Note we need to query the write
// buffer size instead of the #define FLASH_BUF_SIZE
static int intel_bw_program(struct bss_s *bss, uint16 *dst, uint16 *buf, int size)
{
    uint16 status, i;
    uint16 *dst_start;
    int    words2write, result = 0;
    uint32 timeout;

    while ((result == 0) && (size > 0)) {
        words2write = bss->flash_buf_size / 2;

        // Truncate to size
        if (words2write > size)
            words2write = size;
        size -= words2write;
        
        FLASH_WRITE(dst, 0x50); // Intel Clear Status Register Command
        FLASH_WRITE(dst, 0x60); // Intel Config Setup Command
        FLASH_WRITE(dst, 0xD0); // Intel Unlock Block Command

        // wait for buffer to be ready
        do {
            FLASH_WRITE(dst, 0xE8); // buffer program
            status = FLASH_READ(dst);
        } while (!(status & INTEL_STATE_MACHINE_DONE));

        // Write word count - 1
        FLASH_WRITE(dst, words2write - 1); 

        dst_start = dst;

        // Fill buffer
        while (words2write > 0)
        {
            FLASH_WRITE(dst, *buf);
            dst++;
            buf++;
            words2write--;
        }

        // Start the write process
        FLASH_WRITE(dst_start, 0xD0); // buffer program confirm

        // Wait for the write to finish 
        do {
            if ((result = bss->recv(bss)) != 0) {
                FLASH_WRITE(dst_start, 0xFF); // Intel Read Array Command
                return result;
            }
            status = FLASH_READ(dst_start);
        } while (!(status & INTEL_STATE_MACHINE_DONE));

        // Status check
        status = FLASH_READ(dst_start);
        if (status == INTEL_STATE_MACHINE_DONE)
            continue; // No error
        else if (status & INTEL_VPP_RANGE_ERROR)
            result = PROTO_ERROR_FLASH_VPP;
        else if (status & INTEL_PROGRAMMING_ERROR)
            result = PROTO_ERROR_FLASH_TIMEOUT;
        else if (status & INTEL_BLOCK_LOCKED_ERROR)
            result = PROTO_ERROR_FLASH_LOCKED;
        else
            result = 0; // FIXME unknown error

    }
    FLASH_WRITE(dst_start, 0xFF); // Intel Read Array Command

    return result;
}


// Assembly code that satisfies compiler's need of an indirect call
asm("        .state16");
asm("IND$CALL:");
asm("        bx      r4");
