/******************************************************************************
 * FLUID (Flash Loader Utility Independent of Device)
 *
 * (C) Delta Technologies 2001.
 * Cleanup, modifications and extensions by Mads Meisner-Jensen, mmj@ti.com.
 *
 * Mitsubishi Flash Erase and Program Method
 *
 * $Id: mitsubishi.c 1.2 Mon, 21 Oct 2002 18:39:13 +0200 mmj $
 *
 ******************************************************************************/

#include "target.h"

//#define DEBUG_PROGRAM
//#define DEBUG_ERASE


static int mitsubishi_init(uint32 addr);
static int mitsubishi_erase(struct bss_s *bss, uint32 addr);
static int mitsubishi_program(struct bss_s *bss, uint16 *dst, uint16 *buf, int size);

const flash_t my_function_table = {
    mitsubishi_init,
    mitsubishi_erase,
    mitsubishi_program,
    0,
    0,
    0,
    0,
    0xAABBCCDD
};


/******************************************************************************
 * Main
 ******************************************************************************/

static int mitsubishi_init(uint32 addr)
{
    //my_addr = addr;

    return PROTO_READY;
}


// Status bits for Mitsubishi flash memory devices
#define MITSUBISHI_STATE_MACHINE_DONE     (1<<7)
#define MITSUBISHI_BLOCK_ERASE_ERROR      (1<<5)
#define MITSUBISHI_CMD_SEQ_ERROR          ((1<<4)|(1<<5))
#define MITSUBISHI_PROGRAMMING_ERROR      (1<<4)
#define MITSUBISHI_VPP_RANGE_ERROR        (1<<3)
#define MITSUBISHI_BLOCK_LOCKED_ERROR     (1<<1)


static int mitsubishi_erase(struct bss_s *bss, uint32 addr)
{
    int result;
    uint16 status;

#ifdef DEBUG_ERASE
    DEBUG_PUTSTR("E");
    DEBUG_PUTNUM((uint32) addr);
#endif

    FLASH_WRITE(addr, 0x50); // Mitsubishi Clear Status Register Command
    FLASH_WRITE(addr, 0xFF); // Mitsubishi Read Array Command

    FLASH_WRITE(addr, 0x60); // Mitsubishi Config Setup Command
    FLASH_WRITE(addr, 0xD0); // Mitsubishi Unlock Block Command
    FLASH_WRITE(addr, 0xFF); // Mitsubishi Read Array Command

    FLASH_WRITE(addr, 0x20); // Mitsubishi Erase Setup Command
    FLASH_WRITE(addr, 0xD0); // Mitsubishi Erase Confirm Command

    do {
        if ((result = bss->recv(bss)) != 0)
            return result;
        status = FLASH_READ(addr);
    }
    while (!(status & MITSUBISHI_STATE_MACHINE_DONE));

    status = FLASH_READ(addr);

    if (status & MITSUBISHI_VPP_RANGE_ERROR)
        result = PROTO_ERROR_FLASH_VPP;
    else if ((status & MITSUBISHI_CMD_SEQ_ERROR) == MITSUBISHI_CMD_SEQ_ERROR)
        result = PROTO_ERROR_FLASH_COMMAND;
    else if (status & MITSUBISHI_BLOCK_ERASE_ERROR)
        result = PROTO_ERROR_FLASH_TIMEOUT;
    else if (status & MITSUBISHI_BLOCK_LOCKED_ERROR)
        result = PROTO_ERROR_FLASH_LOCKED;
    else
        result = 0;
        
    FLASH_WRITE(addr, 0xFF); // Mitsubishi Read Array Command

#ifdef DEBUG_ERASE
    DEBUG_PUTSTR("S");
    DEBUG_PUTNUM((uint32) status);
    DEBUG_PUTNUM((uint32) result);
#endif

    return result;
}

#define FLASH_PAGE_SIZE 256

static int mitsubishi_program(struct bss_s *bss,
                              uint16 *dst, uint16 *buf, int size)
{
    uint16 status;
    uint16 *dst_start = dst;
    int    i, result = 0;

#ifdef DEBUG_PROGRAM
    DEBUG_PUTSTR("P");
    DEBUG_PUTNUM((uint32) dst);
    DEBUG_PUTNUM((uint32) buf);
    DEBUG_PUTNUM((uint32) size);
#endif

    FLASH_WRITE(dst, 0xFF); // Mitsubishi Read Array Command

    FLASH_WRITE(dst, 0x50); // Mitsubishi Clear Status Register Command
    FLASH_WRITE(dst, 0xFF); // Mitsubishi Read Array Command

    FLASH_WRITE(dst, 0x60); // Mitsubishi Config Setup Command
    FLASH_WRITE(dst, 0xD0); // Mitsubishi Unlock Block Command
    FLASH_WRITE(dst, 0xFF); // Mitsubishi Read Array Command

    while (result == 0 && size > 0)
    {
        FLASH_WRITE(dst, 0x41); // Mitsubishi Program Command

        for (i = 0; i < FLASH_PAGE_SIZE / 2; i++) {
            FLASH_WRITE(&dst[i], buf[i]);
        }

        // Wait for completion. Reading the flash memory while in program
        // mode allows to get the status.
        do {
            if ((result = bss->recv(bss)) != 0)
                return result;
            status = FLASH_READ(dst);
        } while (!(status & MITSUBISHI_STATE_MACHINE_DONE));

        status = FLASH_READ(dst);

        if ((status & MITSUBISHI_CMD_SEQ_ERROR) == MITSUBISHI_CMD_SEQ_ERROR)
            result = PROTO_ERROR_FLASH_COMMAND;
        else if (status & MITSUBISHI_BLOCK_ERASE_ERROR)
            result = PROTO_ERROR_FLASH_TIMEOUT;
        else if (status & MITSUBISHI_PROGRAMMING_ERROR)
            result = PROTO_ERROR_FLASH_TIMEOUT;
        else if (status & MITSUBISHI_VPP_RANGE_ERROR)
            result = PROTO_ERROR_FLASH_VPP;
        else
            result = 0;

        //dst += sizeof(uint16);
        dst  += FLASH_PAGE_SIZE / 2;
        buf  += FLASH_PAGE_SIZE / 2;
        size -= FLASH_PAGE_SIZE / 2;
    }
    FLASH_WRITE(dst_start, 0xFF); // Mitsubishi Read Array Command

    return result;
}

// Assembly code that satisfies compiler's need of an indirect call
asm("        .state16");
asm("IND$CALL:");
asm("        bx      r4");
