#
# FLUID Target makefile
#
# $Id: target.mak 1.23 Mon, 28 Apr 2003 08:49:16 +0200 tsj $
#

DIR := target

# Options:
# 	-mn             normal opt with debug
# 	-me             little endian code
# 	-mt             16 bit state code
# 	-o              optimize (level 2)
# 	-c              no linking (negates -z)
# 	-mw             word align structures
# 	-pwN            warning level N
# 	-g              symbolic debugging
# 	-k              keep .asm file
# 	-i              include path
# 	-fr             output directory
# 	-q              quiet
# 	-ss             C source-code interlist
COPT := -ss -mn -me -mt -o -c -mw -pw2 -i$(DIR) -fr$(DIR) -q
COPT32 := -ss -mn -me -o -c -mw -pw2 -i$(DIR) -fr$(DIR) -q

# The TARGET define is used in lzdecode.c and lzencode.c
COPT += -dTARGET=1
COPT32 += -dTARGET=1

# You don't have to change these variables
#SRCS := $(wildcard $(DIR)/*.c)
#BNAMES := $(basename $(SRCS))
#OBJS := $(addsuffix .obj, $(BNAMES))
#DEPS := $(addsuffix .d, $(BNAMES))


############################################################
# Main Rules
############################################################

ulyboot.out: $(DIR)/ulyboot.obj $(DIR)/ulyboot.cmd
	lnk470 -m $(DIR)/ulyboot.map -o $@ $^

calboot.out: $(DIR)/calboot.obj $(DIR)/calboot.cmd
	lnk470 -m $(DIR)/calboot.map -o $@ $^

calpboot.out: $(DIR)/calpboot.obj $(DIR)/calpboot.cmd
	lnk470 -m $(DIR)/calpboot.map -o $@ $^

$(DIR)/testboot.out: $(DIR)/testboot.obj $(DIR)/boot.cmd
	lnk470 -m $(DIR)/testboot.map -o $@ $^

$(DIR)/cmd.out: $(DIR)/cmd.obj $(DIR)/cmd.cmd
	lnk470 -m $(DIR)/cmd.map -o $@ $^

$(DIR)/cmdp.out: $(DIR)/cmd.obj $(DIR)/certificate.obj $(DIR)/cmdp.cmd
	lnk470 -m $(DIR)/cmdp.map -o $@ $^

$(DIR)/amd.out: $(DIR)/amd.obj $(DIR)/method.cmd
	lnk470 -m $(DIR)/amd.map -o $@ $^

$(DIR)/ram.out: $(DIR)/ram.obj $(DIR)/method.cmd
	lnk470 -m $(DIR)/ram.map -o $@ $^

$(DIR)/sst.out: $(DIR)/sst.obj $(DIR)/method.cmd
	lnk470 -m $(DIR)/sst.map -o $@ $^

$(DIR)/intel.out: $(DIR)/intel.obj $(DIR)/method.cmd
	lnk470 -m $(DIR)/intel.map -o $@ $^

$(DIR)/intel_bw.out: $(DIR)/intel_bw.obj $(DIR)/method.cmd
	lnk470 -m $(DIR)/intel_bw.map -o $@ $^

$(DIR)/mitsubishi.out: $(DIR)/mitsubishi.obj $(DIR)/method.cmd
	lnk470 -m $(DIR)/mitsubishi.map -o $@ $^


############################################################
# Additional dependencies
############################################################

$(DIR)/cmd.obj: $(DIR)/../pc/lzdecode.c


############################################################
# Special Rules
############################################################

# CHIPSET values:
#
# 3 = Ulysses
# 7 = Calypso
# 12 = Calypso Plus
#

# Four special rules for the target/hardware dependent bootloader
$(DIR)/ulyboot.c: $(DIR)/boot.c
	-cp 	$< $@
	-copy $(DIR)\boot.c $(DIR)\ulyboot.c

$(DIR)/calboot.c: $(DIR)/boot.c
	-cp 	$< $@
	-copy $(DIR)\boot.c $(DIR)\calboot.c

$(DIR)/calpboot.c: $(DIR)/boot.c
	-cp 	$< $@
	-copy $(DIR)\boot.c $(DIR)\calpboot.c

$(DIR)/ulyboot.obj: $(DIR)/ulyboot.c
	cl470 -dFLUID_BOOT_STAND_ALONE=1 -dCHIPSET=3 $(COPT) $<

$(DIR)/calboot.obj: $(DIR)/calboot.c
	cl470 -dFLUID_BOOT_STAND_ALONE=1 -dCHIPSET=7 $(COPT) $<

$(DIR)/calpboot.obj: $(DIR)/calpboot.c
	cl470 -dFLUID_BOOT_STAND_ALONE=1 -dCHIPSET=12 $(COPT) $<


# Two special rules for the SST method
$(DIR)/sst.c: $(DIR)/amd.c
	-cp 	$< $@
	-copy $(DIR)\amd.c $(DIR)\sst.c

$(DIR)/sst.obj: $(DIR)/sst.c $(DIR)/target.h
	cl470 -dSST=1 $(COPT) $<


# Two special rules for the bootloader test program (only for Ulysses)
$(DIR)/testboot.c: $(DIR)/boot.c
	-cp 	$< $@
	-copy $(DIR)\boot.c $(DIR)\testboot.c

$(DIR)/testboot.obj: $(DIR)/testboot.c $(TARGET_HEADERS)
	cl470 -dFLUID_BOOT_STAND_ALONE=1 -dCHIPSET=3 -dTESTBOOT=1 $(COPT) $<


# Special rule for Secure Calypso Plus
$(DIR)/certificate.obj: $(DIR)/certificate.c
	cl470 $(COPT32) $<


############################################################
# Generic Rules
############################################################

%.m0: $(DIR)/%.out
	hex470 -m -memwidth 16 -romwidth 16 -o $@ $^

$(DIR)/%.obj: $(DIR)/%.c $(DIR)/target.h
	cl470 $(COPT) $<


#-include $(DEPS)

